@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.201.2.6 2004/12/03 04:15:29 carpaski Exp $
+# $Header: /var/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.201.2.24 2005/02/26 11:22:37 carpaski Exp $
 
 export SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/usr/lib/portage/pym:/dev/random"
 export SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:${PORTAGE_TMPDIR}"
@@ -90,6 +90,8 @@ addpredict()
 save_IFS
 IFS=$'\n'
 for dir in ${PROFILE_PATHS}; do
+	# Must unset it so that it doesn't mess up assumptions in the RCs.
+	unset IFS
 	if [ -f "${dir}/profile.bashrc" ]; then
 		source "${dir}/profile.bashrc"
 	fi
@@ -347,6 +349,8 @@ unpack() {
 		tarvars="--no-same-owner"	
 	fi	
 
+	[ -z "$*" ] && die "Nothing passed to the 'unpack' command"
+
 	for x in "$@"; do
 		myfail="failure unpacking ${x}"
 		echo ">>> Unpacking ${x} to $(pwd)"
@@ -387,17 +391,25 @@ unpack() {
 	done
 }
 
+strip_duplicate_slashes () { 
+	if [ -n "${1}" ]; then
+		local removed="${1/\/\///}"
+		[ "${removed}" != "${removed/\/\///}" ] && removed=$(strip_duplicate_slashes "${removed}")
+		echo ${removed}
+	fi
+}
+
 econf() {
 	if [ -z "${ECONF_SOURCE}" ]; then
 		ECONF_SOURCE="."
 	fi
 	if [ -x "${ECONF_SOURCE}/configure" ]; then
 		if hasq autoconfig $FEATURES && ! hasq autoconfig $RESTRICT; then
-			if [ -e /usr/share/gnuconfig/ -a -x /bin/basename ]; then
+			if [ -e /usr/share/gnuconfig/ ]; then
 				local x
-				for x in $(find ${S} -type f -name config.guess -o -name config.sub) ; do
-					einfo "econf: updating $x with /usr/share/gnuconfig/$(/bin/basename ${x})"
-					cp /usr/share/gnuconfig/$(/bin/basename ${x}) ${x}
+				for x in $(find ${WORKDIR} -type f -name config.guess -o -name config.sub) ; do
+					echo " * econf: updating ${x/${WORKDIR}\/} with /usr/share/gnuconfig/${x##*/}"
+					cp /usr/share/gnuconfig/${x##*/} ${x}
 				done
 			fi
 		fi
@@ -412,16 +424,35 @@ econf() {
 		
 		# if the profile defines a location to install libs to aside from default, pass it on.
 		# if the ebuild passes in --libdir, they're responsible for the conf_libdir fun.
-		if [ ! -z "${CONF_LIBDIR}" ] && [ "${*/--libdir}" == "$*" ]; then
-			if [ "${*/--prefix}" == "$*" ]; then
-				CONF_PREFIX="/usr"
-			else
+		LIBDIR_VAR="LIBDIR_${ABI}"
+		if [ -n "${ABI}" -a -n "${!LIBDIR_VAR}" ]; then
+			CONF_LIBDIR="${!LIBDIR_VAR}"
+		fi
+		unset LIBDIR_VAR
+		if [ -n "${CONF_LIBDIR}" ] && [ "${*/--libdir}" == "$*" ]; then
+			if [ "${*/--exec-prefix}" != "$*" ]; then
+				local args="$(echo $*)"
+				local -a pref=($(echo ${args/*--exec-prefix[= ]}))
+				CONF_PREFIX=${pref}
+				[ "${CONF_PREFIX:0:1}" != "/" ] && CONF_PREFIX="/${CONF_PREFIX}"
+			elif [ "${*/--prefix}" != "$*" ]; then
 				local args="$(echo $*)"
 				local -a pref=($(echo ${args/*--prefix[= ]}))
 				CONF_PREFIX=${pref}
+				[ "${CONF_PREFIX:0:1}" != "/" ] && CONF_PREFIX="/${CONF_PREFIX}"
+			else
+				CONF_PREFIX="/usr"
 			fi
 			export CONF_PREFIX
-			EXTRA_ECONF="--libdir=/${CONF_PREFIX}/${CONF_LIBDIR} ${EXTRA_ECONF}"
+			[ "${CONF_LIBDIR:0:1}" != "/" ] && CONF_LIBDIR="/${CONF_LIBDIR}"
+
+			CONF_LIBDIR_RESULT="${CONF_PREFIX}${CONF_LIBDIR}"
+			for X in 1 2 3; do
+				# The escaping is weird. It will break if you escape the last one.
+				CONF_LIBDIR_RESULT="${CONF_LIBDIR_RESULT//\/\///}"
+			done
+
+			EXTRA_ECONF="--libdir=${CONF_LIBDIR_RESULT} ${EXTRA_ECONF}"
 		fi
 		
 		echo "${ECONF_SOURCE}/configure" \
@@ -435,7 +466,7 @@ econf() {
 			${EXTRA_ECONF} \
 			"$@"
 
-		"${ECONF_SOURCE}/configure" \
+		if ! "${ECONF_SOURCE}/configure" \
 			--prefix=/usr \
 			--host=${CHOST} \
 			--mandir=/usr/share/man \
@@ -444,7 +475,15 @@ econf() {
 			--sysconfdir=/etc \
 			--localstatedir=/var/lib \
 			${EXTRA_ECONF} \
-			"$@" || die "econf failed"
+			"$@" ; then
+
+			if [ -s config.log ]; then
+				echo
+				echo "!!! Please attach the config.log to your bug report:"
+				echo "!!! ${PWD}/config.log"
+			fi
+			die "econf failed"
+		fi
 	else
 		die "no configure script found"
 	fi
@@ -452,9 +491,18 @@ econf() {
 
 einstall() {
 	# CONF_PREFIX is only set if they didn't pass in libdir above.
-	if [ ! -z "${CONF_LIBDIR}" ] && [ "${CONF_PREFIX:-unset}" != "unset" ]; then
-		EXTRA_EINSTALL="libdir=${D}/${CONF_PREFIX}/${CONF_LIBDIR} ${EXTRA_EINSTALL}"
+	LIBDIR_VAR="LIBDIR_${ABI}"
+	if [ -n "${ABI}" -a -n "${!LIBDIR_VAR}" ]; then
+		CONF_LIBDIR="${!LIBDIR_VAR}"
 	fi
+	unset LIBDIR_VAR
+	if [ -n "${CONF_LIBDIR}" ] && [ "${CONF_PREFIX:-unset}" != "unset" ]; then
+		EI_DESTLIBDIR="${D}/${CONF_PREFIX}/${CONF_LIBDIR}"
+		EI_DESTLIBDIR="$(strip_duplicate_slashes ${EI_DESTLIBDIR})"
+		EXTRA_EINSTALL="libdir=${EI_DESTLIBDIR} ${EXTRA_EINSTALL}"
+		unset EI_DESTLIBDIR
+	fi
+
 	if [ -f ./[mM]akefile -o -f ./GNUmakefile ] ; then
 		if [ ! -z "${PORTAGE_DEBUG}" ]; then
 			make -n prefix=${D}/usr \
@@ -515,14 +563,14 @@ src_test()
 	if make check -n &> /dev/null; then
 		echo ">>> Test phase [check]: ${CATEGORY}/${PF}"
 		if ! make check; then
-			hasq maketest $FEATURES && die "Make check failed. See above for details."
-			hasq maketest $FEATURES || eerror "Make check failed. See above for details."
+			hasq test $FEATURES && die "Make check failed. See above for details."
+			hasq test $FEATURES || eerror "Make check failed. See above for details."
 		fi
 	elif make test -n &> /dev/null; then
 		echo ">>> Test phase [test]: ${CATEGORY}/${PF}"
 		if ! make test; then
-			hasq maketest $FEATURES && die "Make test failed. See above for details."
-			hasq maketest $FEATURES || eerror "Make test failed. See above for details."
+			hasq test $FEATURES && die "Make test failed. See above for details."
+			hasq test $FEATURES || eerror "Make test failed. See above for details."
 		fi
 	else
 		echo ">>> Test phase [none]: ${CATEGORY}/${PF}"
@@ -638,18 +686,29 @@ dyn_unpack() {
 }
 
 dyn_clean() {
+	if [ "$USERLAND" == "BSD" ] && type -p chflags &>/dev/null; then
+		chflags -R noschg,nouchg,nosappnd,nouappnd,nosunlnk,nouunlnk \
+			"${BUILDDIR}"
+	fi
+	
+	if [ "$USERLAND" == "Darwin" ] && type -p chflags &>/dev/null; then
+		chflags -R noschg,nouchg,nosappnd,nouappnd "${BUILDDIR}"
+	fi
+
 	rm -rf "${BUILDDIR}/image"
 
 	if ! hasq keeptemp $FEATURES; then
-		rm -rf "${T}"/*
+		rm -rf "${T}"
 	else
 		mv "${T}/environment" "${T}/environment.keeptemp"
 	fi
 
 	if ! hasq keepwork $FEATURES; then
-		rm -rf "${BUILDDIR}/.compiled"
 		rm -rf "${BUILDDIR}/.unpacked"
+		rm -rf "${BUILDDIR}/.compiled"
+		rm -rf "${BUILDDIR}/.tested"
 		rm -rf "${BUILDDIR}/.installed"
+		rm -rf "${BUILDDIR}/.packaged"
 		rm -rf "${BUILDDIR}/build-info"
 		rm -rf "${WORKDIR}"
 	fi
@@ -657,6 +716,10 @@ dyn_clean() {
 	if [ -f "${BUILDDIR}/.unpacked" ]; then
 		find "${BUILDDIR}" -type d ! -regex "^${WORKDIR}" | sort -r | tr "\n" "\0" | $XARGS -0 rmdir &>/dev/null
 	fi
+
+	if [ -z "$(find "${BUILDDIR}" -mindepth 1 -maxdepth 1)" ]; then
+		rmdir "${BUILDDIR}"
+	fi
 	true
 }
 
@@ -925,12 +988,13 @@ dyn_test() {
 	if [ -d "${S}" ]; then
 		cd "${S}"
 	fi
-	if hasq maketest $RESTRICT; then
+	if hasq maketest $RESTRICT || hasq test $RESTRICT; then
 		ewarn "Skipping make test/check due to ebuild restriction."
 		echo ">>> Test phase [explicitly disabled]: ${CATEGORY}/${PF}"
-	elif ! hasq maketest $FEATURES; then
+	elif ! hasq test $FEATURES; then
 		echo ">>> Test phase [not enabled]: ${CATEGORY}/${PF}"
 	else
+		echo ">>> Test phase [enabled]: ${CATEGORY}/${PF}"
 		src_test
 	fi
 
@@ -982,7 +1046,7 @@ dyn_install() {
 						#UNSAFE=$(($UNSAFE + 1))
 						echo -ne '\a'
 						echo "QA Notice: ${x:${#D}:${#x}} is setXid, dynamically linked and using lazy bindings."
-						echo "This combination is generally discouraged. Try: LDFLAGS='-Wl,-z,now' emerge ${PN}"
+						echo "This combination is generally discouraged. Try: CFLAGS='-Wl,-z,now' emerge ${PN}"
 						echo -ne '\a'
 						sleep 1
 					fi
@@ -997,27 +1061,57 @@ dyn_install() {
 	
 	function stat_perms() {
 		local f
-		f=$(stat -c '%f' "$1")
-		f=$(printf %o 0x$f)
-		f="${f:${#f}-4}"
+		if ! type -p stat &>/dev/null; then
+			do_stat() {
+				# Generic version -- Octal result
+				python -c "import os,stat; print '%o' % os.stat('$1')[stat.ST_MODE]"
+			}
+		else
+			if [ "${USERLAND}" == "BSD" ] || [ "${USERLAND}" == "Darwin" ]; then
+				do_stat() {
+					# BSD version -- Octal result
+					$(type -p stat) -f '%p' "$1"
+				}
+			else
+				do_stat() {
+					# Linux version -- Hex result converted to Octal
+					f=$($(type -p stat) -c '%f' "$1") || return $?
+					printf '%o' "0x$f"
+				}
+			fi
+		fi
+
+		f=$(do_stat "$@") || die "Failed to perform stat operation on file: $1"
+		f="${f:2:4}"
 		echo $f
 	}
 
 	local file s
+	local count=0
 	find "${D}/" -user  portage | while read file; do
-		ewarn "file $file was installed with user portage!"
+		count=$(( $count + 1 ))
 		s=$(stat_perms $file)
 		chown root "$file"
 		chmod "$s" "$file"
 	done
+	if (( $count > 0 )); then
+		ewarn "$count files were installed with user portage!"
+	fi
 
+	count=0
 	find "${D}/" -group portage | while read file; do
-		ewarn "file $file was installed with group portage!"
+		count=$(( $count + 1 ))
 		s=$(stat_perms "$file")
-		[ "$USERLAND" == "BSD" ] && chgrp wheel "$file"
-		[ "$USERLAND" != "BSD" ] && chgrp root "$file"
+		if [ "$USERLAND" == "BSD" ] || [ "$USERLAND" == "Darwin" ];then
+			chgrp wheel "$file"
+		else
+			chgrp root "$file"
+		fi
 		chmod "$s" "$file"
 	done
+	if (( $count > 0 )); then
+		ewarn "$count files were installed with group portage!"
+	fi
 
 	unset -f stat_perms
 
@@ -1026,6 +1120,17 @@ dyn_install() {
 		rm -f "${D}/usr/share/info/dir.gz"
 	fi
 
+	if hasq multilib-strict ${FEATURES} && [ -x /usr/bin/file -a -x /usr/bin/find -a \
+	     -n "${MULTILIB_STRICT_DIRS}" -a -n "${MULTILIB_STRICT_DENY}" ]; then
+		MULTILIB_STRICT_EXEMPT=${MULTILIB_STRICT_EXEMPT:-"(perl5|gcc|gcc-lib)"}
+		for dir in ${MULTILIB_STRICT_DIRS}; do
+			[ -d "${D}/${dir}" ] || continue
+			for file in $(find ${D}/${dir} -type f | egrep -v "^${D}/${dir}/${MULTILIB_STRICT_EXEMPT}"); do
+				file ${file} | egrep -q "${MULTILIB_STRICT_DENY}" && die "File ${file} matches a file type that is not allowed in ${dir}"
+			done
+		done
+	fi
+
 	touch "${BUILDDIR}/.installed"
 	echo ">>> Completed installing ${PF} into ${D}"
 	echo
@@ -1040,6 +1145,23 @@ dyn_preinst() {
 
 	pkg_preinst
 
+	# hopefully this will someday allow us to get rid of the no* feature flags
+	# we don't want globbing for initial expansion, but afterwards, we do
+	local shopts=$-
+	set -o noglob
+	for no_inst in ${INSTALL_MASK}; do
+		set +o noglob
+		einfo "Removing ${no_inst}"
+		# normal stuff
+		rm -Rf ${IMAGE}/${no_inst} >&/dev/null
+
+		# we also need to handle globs (*.a, *.h, etc)
+		find "${IMAGE}" -name ${no_inst} -exec rm -fR {} \; >&/dev/null
+	done
+	# set everything back the way we found it
+	set +o noglob
+	set -${shopts}
+
 	# remove man pages
 	if hasq noman $FEATURES; then
 		rm -fR "${IMAGE}/usr/share/man"
@@ -1487,7 +1609,7 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 	cd ${PORTAGE_TMPDIR} &> /dev/null
 	cd ${BUILD_PREFIX} &> /dev/null
 
-	if [ `id -nu` == "portage" ] ; then
+	if [ "$(id -nu)" == "portage" ] ; then
 		export USER=portage
 	fi
 
